use super::models;
use std::collections::HashMap;

///
pub struct Repository {
  client: reqwest::Client,
  k_secret: String,
  k_username: String,
}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of api.kgon.identity").finish()
  }
}

impl Repository {
  ///
  pub fn new(k_secret: &str, k_username: &str) -> Repository {
    Repository {
      client: reqwest::Client::new(),
      k_secret: k_secret.to_string(),
      k_username: k_username.to_string(),
    }
  }

  ///
  pub async fn find_balance(
    &self,
    data: models::FindBalanceRequest,
  ) -> Result<models::FindBalanceResponse, models::Error> {
    let mut params = HashMap::new();

    params.insert("username", data.username);

    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/balance")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::FindBalanceResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }

  ///
  pub async fn find_partner_balance(
    &self,
    data: models::FindPartnerBalanceRequest,
  ) -> Result<models::FindPartnerBalanceResponse, models::Error> {
    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/partner/balance")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::FindPartnerBalanceResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }

  ///
  pub async fn save_deposit(
    &self,
    data: models::SaveDepositRequest,
  ) -> Result<models::SaveDepositResponse, models::Error> {
    let mut params = HashMap::new();

    params.insert("username", data.username);
    if let Some(cash_type) = data.cash_type {
      params.insert("cashType", cash_type);
    }
    params.insert("amount", data.amount.to_string());
    if let Some(request_key) = data.request_key {
      params.insert("requestKey", request_key);
    }

    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/deposit")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::SaveDepositResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }

  ///
  pub async fn save_withdraw(
    &self,
    data: models::SaveWithdrawRequest,
  ) -> Result<models::SaveWithdrawResponse, models::Error> {
    let mut params = HashMap::new();

    params.insert("username", data.username);
    if let Some(request_key) = data.request_key {
      params.insert("requestKey", request_key);
    }

    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/withdraw")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::SaveWithdrawResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }
}
