use super::models;
use std::{collections::HashMap, time::Duration};

///
pub struct Repository {
  client: reqwest::Client,
  k_secret: String,
  k_username: String,
}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of api.kgon.identity").finish()
  }
}

impl Repository {
  ///
  pub fn new(k_secret: &str, k_username: &str) -> Repository {
    Repository {
      client: reqwest::Client::new(),
      k_secret: k_secret.to_string(),
      k_username: k_username.to_string(),
    }
  }

  ///
  pub async fn find_vendors(
    &self,
    data: models::FindVendorsRequest,
  ) -> Result<models::FindVendorsResponse, models::Error> {
    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/vendors")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::FindVendorsResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }

  ///
  pub async fn find_games(
    &self,
    data: models::FindGamesRequest,
  ) -> Result<models::FindGamesResponse, models::Error> {
    let mut params = HashMap::new();

    params.insert("vendorKey", data.vendor_key);

    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/games")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::FindGamesResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }

  ///
  pub async fn play(
    &self,
    data: models::PlayRequest,
  ) -> Result<models::PlayResponse, models::Error> {
    let mut params = HashMap::new();

    params.insert("vendorKey", data.vendor_key);
    params.insert("gameKey", data.game_key);
    params.insert("username", data.username);
    params.insert("nickname", data.nickname);
    params.insert("siteUsername", data.site_username);

    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }

    params.insert("amount", data.amount.to_string());

    if let Some(request_key) = data.request_key {
      params.insert("requestKey", request_key);
    }

    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/play")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .timeout(Duration::from_secs(10))
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::PlayResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }
}
